"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFormTemplate = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const listFiles_1 = require("../utils/listFiles");
const core_1 = require("@angular-devkit/core");
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const change_1 = require("@schematics/angular/utility/change");
const updateFile_1 = require("../utils/updateFile");
const utils_1 = require("../utils/utils");
let importsToAdd = [
    {
        packageName: "@windmillcode/angular-wml-form",
        importStatement: "import { WMLForm } from '@windmillcode/angular-wml-form';\n",
        importMembers: ["WMLForm"]
    },
    {
        packageName: "@windmillcode/angular-wml-button-zero",
        importStatement: "import { WmlButtonOneParams, WMLButtonParamsTypeEnum } from '@windmillcode/angular-wml-button-zero';\n",
        importMembers: ["WmlButtonOneParams", "WMLButtonParamsTypeEnum"]
    },
    {
        packageName: "@windmillcode/angular-wml-components-base",
        importStatement: "import { WMLUIProperty, generateClassPrefix } from '@windmillcode/angular-wml-components-base';\n",
        importMembers: ["WMLUIProperty", "generateClassPrefix"]
    },
    {
        packageName: "@core/base/base.service",
        importStatement: "import { BaseService, BaseServiceSubmitFormParams } from '@core/base/base.service';\n",
        importMembers: ["BaseService", "BaseServiceSubmitFormParams"]
    },
    // {
    //   packageName: "@core/utility/form-utils",
    //   importStatement: "import { createFieldString , createI18nErrorMsgsBasedOnValidations } from '@core/utility/form-utils';\n",
    //   importMembers: ["createFieldString","createI18nErrorMsgsBasedOnValidations"]
    // },
    {
        packageName: "@shared/services/forms/forms.service",
        importStatement: "import { FormsService } from '@shared/services/forms/forms.service';\n",
        importMembers: ["FormsService"]
    },
    {
        packageName: '@env/environment',
        importStatement: `import { ENV } from '@env/environment';`,
        importMembers: ["ENV"]
    },
];
class TemplateFormSchema {
    constructor() {
        this.envFilePath = utils_1.DevEnvFile;
        this.formsServiceFilePath = utils_1.FormsServiceFilePath;
    }
}
function generateFormTemplate(options) {
    return () => {
        return (0, schematics_1.chain)([
            updateClassFile(options)
        ]);
    };
}
exports.generateFormTemplate = generateFormTemplate;
let updateClassFile = (options) => {
    return (tree) => {
        let files = (0, listFiles_1.listFiles)(tree, options.path);
        let targetFiles = [
            { suffix: "component.ts", predicate: updateCpntFile },
            { suffix: "service.ts", predicate: updateCpntFile },
            { suffix: "directive.ts", predicate: updateCpntFile },
        ];
        let targetFileName = files.find((file) => {
            return targetFiles.some((val) => file.endsWith(val.suffix));
        });
        let targetFileInfo = targetFiles.find((val) => {
            return targetFileName.endsWith(val.suffix);
        });
        let targetFilePath = (0, core_1.normalize)(`${targetFileName}`);
        targetFileInfo === null || targetFileInfo === void 0 ? void 0 : targetFileInfo.predicate(options, tree, targetFilePath);
    };
};
function updateCpntFile(options, tree, targetFilePath) {
    var _a;
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let cpntClassVar = (0, utils_1.getCpntClassVar)(sourceFile);
    let cpntClassName = (_a = cpntClassVar.name) === null || _a === void 0 ? void 0 : _a.getText();
    if (!cpntClassName) {
        throw new schematics_1.SchematicsException("component class name is missing or misspelt or the script is having issues finding the component class name");
    }
    let lowerCamelCaseCpntClassName = schematics_1.strings.camelize(cpntClassName.split("Component")[0]);
    let classNamePrefix = cpntClassName.split("Component")[0];
    let apiCallArgument = ![null, undefined, ""].includes(options.apiCallClass) ?
        `new ${options.apiCallClass}(this.${options.name}Group.value)` :
        `this.${options.name}Group.value`;
    let lastProperty = cpntClassVar.members
        .filter(ts.isPropertyDeclaration)
        .at(-1);
    let nameString = (0, utils_1.createStringObject)(options.name, "");
    let cpntString = (0, utils_1.createStringObject)(cpntClassName, "Component");
    injectSnippet(options, lowerCamelCaseCpntClassName, lastProperty, targetFilePath, tree, classNamePrefix, apiCallArgument, nameString);
    addInitFormCallToNgOnInit(targetFilePath, tree, options);
    (0, utils_1.performDependencyInjection)(targetFilePath, tree, `public formsService:FormsService`, (0, utils_1.getCpntClassVar)(sourceFile));
    (0, utils_1.addImportStatements)(sourceFile, targetFilePath, tree, importsToAdd);
    updateFormsServiceFile(tree, lowerCamelCaseCpntClassName, options);
    updateEnJSONFile(tree, classNamePrefix, options);
    updateSpecFile(options, tree, cpntString.dasherize() + ".component.spec.ts", nameString, cpntString);
    // sourceFile = wmlCreateSourceFile(targetFilePath, tree);
    // console.log(sourceFile.getFullText())
    // throw new SchematicsException("got here")
}
function injectSnippet(options, lowerCamelCaseCpntClassName, lastProperty, targetFilePath, tree, classNamePrefix, apiCallArgument, nameString) {
    var _a;
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let cpntClassVar = (0, utils_1.getCpntClassVar)(sourceFile);
    let insertPos = 0;
    let final = "";
    let fieldSnippets;
    if (options.fieldType === "complex") {
        fieldSnippets = createComplexFieldSnippets(options, lowerCamelCaseCpntClassName, classNamePrefix);
    }
    else if (options.fieldType === "simple") {
        fieldSnippets = createSimpleFieldSnippets(options, lowerCamelCaseCpntClassName, nameString);
    }
    let fnBodySnippet = fieldSnippets.map((x) => x.fnBody).join("\n");
    let fnNameSnippet = fieldSnippets.map((x) => `\t\t_add_newline_here_this.${x.fnName}(),`).join("");
    let optionString = (0, utils_1.createStringObject)(options.name);
    let submitBtnObjSnippet = `${options.name}SubmitBtn = new WmlButtonOneParams({
  id:this.idPrefix("${options.name}SubmitBtn"),
  type:WMLButtonParamsTypeEnum.PRIMARY,
  text:"global.submitBtn",
  click:()=> ["PREVIEW","PROD"].includes(ENV.type) ?
  this.baseService.openFeatureIsComingSoon() : this.${options.name}ClickSubmitBtn()
})
`.replace(/\n/g, "_add_newline_here_");
    let submitBtnSnippet = [null, undefined, ""].includes(options.apiCallClass) ? `_add_newline_here_

${submitBtnObjSnippet}

_add_newline_here_
${options.name}ClickSubmitBtn = ()=>{
_add_newline_here_
    this.baseService.submitForm(new BaseServiceSubmitFormParams({
      rootFormGroup:this.${options.name}Group,
      cdref:this.cdref,
      validFormPredicateTypeDefault:{
        ngUnsub:this.ngUnsub,
        // @ts-ignore
        apiCall$:${options.apiCall}(${apiCallArgument})
      }
    }))
_add_newline_here_
}` : ``;
    let simpleFieldsSnippet = options.fieldType === "simple" ? `
    create${nameString.capitalize(false)}Field = (params:Partial<{createWMLFieldFn:any,errorMsgKeyArray:any,fieldCustomParams:any,formControlName: keyof  FormsService["${lowerCamelCaseCpntClassName}"]["mainForm"]["value"]}>)=>{
      let {createWMLFieldFn,errorMsgKeyArray,fieldCustomParams,formControlName} = params
      let field =this.baseService.createWMLField({
        i18nPrefix:"${classNamePrefix}.${options.name}",
        idPrefixFn:this.idPrefix,
        fieldParentForm:this.${options.name}Group,
        formControlName,createWMLFieldFn,errorMsgKeyArray,fieldCustomParams
      })
      return field
    }
  ` : ``;
    let snippet = `
    ${options.name} = new WMLForm({})
    ${options.name}Group = this.formsService.${lowerCamelCaseCpntClassName}.${options.name}
_add_newline_here_
    ${simpleFieldsSnippet}
${submitBtnSnippet}
${fnBodySnippet}
_add_newline_here_
    init${optionString.capitalize(false)} = ()=>{
      let fields = [
        ${fnNameSnippet}
      ]
      this.${options.name}.updateFields(fields)
    }
  `;
    let text = sourceFile.getFullText();
    let indentation = (0, utils_1.getIndentation)(text);
    if (!lastProperty) {
        let constructor = (0, utils_1.wmlFindChildNode)(cpntClassVar.members, ts.SyntaxKind.Constructor, "something is wrong with the components constructor", false, false);
        if (!constructor) {
            let openBrace = (0, utils_1.wmlFindChildNode)(cpntClassVar, ts.SyntaxKind.OpenBraceToken, "its with classes open brace token is it missing");
            insertPos = openBrace === null || openBrace === void 0 ? void 0 : openBrace.end;
        }
        else {
            insertPos = constructor === null || constructor === void 0 ? void 0 : constructor.end;
        }
        snippet = (0, utils_1.removeLeadingWhitespace)(snippet, "\t\t\t\t\n");
    }
    else {
        insertPos = lastProperty.end;
        snippet = (0, utils_1.removeLeadingWhitespace)(snippet, lastProperty.getFullText());
    }
    final = `${(_a = indentation[0]) !== null && _a !== void 0 ? _a : ' '}${snippet}`;
    let changes = [new change_1.InsertChange(targetFilePath, insertPos, final)];
    (0, updateFile_1.wmlUpdateFile)(tree, targetFilePath, changes);
}
function addInitFormCallToNgOnInit(targetFilePath, tree, options) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let cpntClassVar = (0, utils_1.getCpntClassVar)(sourceFile);
    let optionString = (0, utils_1.createStringObject)(options.name);
    let ngOnInit = cpntClassVar.members
        .filter((x) => {
        return x.kind === ts.SyntaxKind.MethodDeclaration;
    })
        .find((x) => {
        var _a;
        return ((_a = x.name) === null || _a === void 0 ? void 0 : _a.getText()) === "ngOnInit";
    });
    if (!ngOnInit) {
        let constructor = (0, utils_1.wmlFindChildNode)(cpntClassVar.members, ts.SyntaxKind.Constructor, "its with the constructor", false, false);
        let ngOnInitStr = `\n
    ngOnInit(): void{
      this.init${optionString.capitalize(false)}() _add_newline_here_
    }
    `;
        if (!constructor) {
            let openBrace = (0, utils_1.wmlFindChildNode)(cpntClassVar, ts.SyntaxKind.OpenBraceToken, "its with component class close {", false, false);
            (0, utils_1.prepareForUpdate)(ngOnInitStr, targetFilePath, tree, openBrace.end, cpntClassVar.getFullText());
        }
        else {
            let openParenToken = (0, utils_1.wmlFindChildNode)(constructor, ts.SyntaxKind.OpenParenToken, "its with constuctor close parentheses", false, false);
            (0, utils_1.prepareForUpdate)(ngOnInitStr, targetFilePath, tree, openParenToken.end, constructor.getFullText());
        }
    }
    else {
        let block = (0, utils_1.wmlFindChildNode)(ngOnInit, ts.SyntaxKind.Block, "its with ngOnInit's block");
        let openBrace = (0, utils_1.wmlFindChildNode)(block, ts.SyntaxKind.OpenBraceToken, "its with ngOnInit's open brace");
        (0, utils_1.prepareForUpdate)(`\nthis.init${optionString.capitalize(false)}() _add_newline_here_`, targetFilePath, tree, openBrace.end, ngOnInit.getFullText(), "\t\t");
    }
}
let updateFormsServiceFile = (tree, lowerCamelCaseCpntClassName, options) => {
    var _a;
    let { targetPath } = (0, utils_1.getFile)(tree, utils_1.FormsServiceFilePath);
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetPath, tree);
    let targetClass = (_a = sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.statements) === null || _a === void 0 ? void 0 : _a.filter(ts.isClassDeclaration).find((v) => {
        var _a;
        return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === 'FormsService';
    });
    let insertPos = 0;
    let snippet = "";
    let indentTarget;
    let spacingPrefix = "";
    let fieldsArray = options.fields.map((val) => {
        return `${val}:new FormControl("")`;
    });
    let fieldsString = "_add_newline_here_\t" + fieldsArray.join(",_add_newline_here_\t") + "_add_newline_here_";
    let formBlock = `${options.name} :new FormGroup({${fieldsString}})`;
    let entityProp = (0, utils_1.wmlFindChildNode)(targetClass.members, 
    // @ts-ignore
    (prop) => {
        var _a;
        let name = (_a = prop.name) === null || _a === void 0 ? void 0 : _a.getText();
        return name === lowerCamelCaseCpntClassName;
    }, "its with the properties of FormsService", false, false);
    if (!entityProp) {
        let constructor = (0, utils_1.wmlFindChildNode)(targetClass.members, 
        // @ts-ignore
        ts.SyntaxKind.Constructor, "look like the constructor", false, false);
        snippet = `
    ${lowerCamelCaseCpntClassName}={
      ${formBlock}
    }
    `;
        spacingPrefix = "\n";
        if (!constructor) {
            let openBrace = (0, utils_1.wmlFindChildNode)(targetClass, ts.SyntaxKind.OpenBraceToken, "its with FormService Class open brace token is it missing");
            indentTarget = targetClass;
            insertPos = openBrace.end;
        }
        else {
            indentTarget = constructor;
            insertPos = constructor.end;
        }
    }
    else {
        fieldsString = "_add_newline_here_\t" + fieldsArray.join(",_add_newline_here_\t") + "_add_newline_here_\t";
        formBlock = `${options.name} :new FormGroup({${fieldsString}})`;
        let objLiteral = (0, utils_1.wmlFindChildNode)(entityProp, ts.SyntaxKind.ObjectLiteralExpression, "its with the braces on  the " + entityProp + " property");
        let closeBrace = (0, utils_1.wmlFindChildNode)(objLiteral, ts.SyntaxKind.CloseBraceToken, "its with the braces on  the " + entityProp + " property");
        indentTarget = closeBrace;
        insertPos = closeBrace.pos;
        let occurencess = objLiteral.properties.length;
        snippet = "\n" + formBlock;
        if (occurencess > 0) {
            let lastRouteLiteral = objLiteral.properties.at(-1);
            indentTarget = lastRouteLiteral;
            insertPos = lastRouteLiteral.end;
            spacingPrefix = ",";
        }
    }
    (0, utils_1.prepareForUpdate)(snippet, targetPath, tree, insertPos, 
    // @ts-ignore
    indentTarget === null || indentTarget === void 0 ? void 0 : indentTarget.getFullText(), spacingPrefix);
};
let updateEnJSONFile = (tree, classNamePrefix, options) => {
    var _a;
    let { targetPath } = (0, utils_1.getFile)(tree, utils_1.i18nEnJSONFilePath);
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetPath, tree);
    let enJSON = JSON.parse(sourceFile.getFullText());
    enJSON[classNamePrefix] = (_a = enJSON[classNamePrefix]) !== null && _a !== void 0 ? _a : {};
    let fieldsArray = options.fields.map((field) => {
        return [
            field + "Field",
            {
                "label": (0, utils_1.camelCaseToSentence)(schematics_1.strings.capitalize(field)),
                "error": {}
            }
        ];
    });
    enJSON[classNamePrefix][options.name] = Object.fromEntries(fieldsArray);
    tree.overwrite(utils_1.i18nEnJSONFilePath, JSON.stringify(enJSON, null, 2));
};
function createSimpleFieldSnippets(options, lowerCamelCaseCpntClassName, nameString) {
    return options.fields.map((field) => {
        let fnName = `create${schematics_1.strings.capitalize(field)}Field`;
        return {
            fnName,
            fnBody: `_add_newline_here_
    ${fnName} =()=>{
      let field= this.create${nameString.capitalize(false)}Field({
        formControlName:"${field}",
        errorMsgKeyArray:[],
        fieldCustomParams:null
      })
      field.deleteRequiredLabelPart();
      return field
    }`
        };
    });
}
function createComplexFieldSnippets(options, lowerCamelCaseCpntClassName, classNamePrefix) {
    return options.fields.map((field) => {
        let fnName = `create${schematics_1.strings.capitalize(field)}Field`;
        return {
            fnName,
            fnBody: `_add_newline_here_
    ${fnName} =()=>{
      let val = createFieldString(ENV.${lowerCamelCaseCpntClassName}.${options.name}.${field !== null && field !== void 0 ? field : "name"}FormControlName)
      let field = this.baseService.createInputFormField({
        id:this.idPrefix(val.field),
        labelValue:"${classNamePrefix}.${options.name}."+val.field+".label",
        fieldFormControlName:this.${options.name}Vars[val.base+"FormControlName"],
        fieldParentForm:this.${options.name}Group,
        selfType:"standalone",
        errorMsgs:createI18nErrorMsgsBasedOnValidations(
          "${classNamePrefix}.${options.name}",val.field,["replace","with","your","error","keys"]
        ),
      })
      field.deleteRequiredLabelPart()
      return field
    }`
        };
    });
}
function updateSpecFile(options, tree, targetFileName, nameString, cpntString) {
    let { targetPath } = (0, utils_1.getFile)(tree, `${options.path}/${targetFileName}`);
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetPath, tree);
    addNeededImportsToTestFile(sourceFile, targetPath, tree);
    sourceFile = addDescribeBlockToCpntSpecFile(targetPath, tree, options, nameString, cpntString);
    sourceFile = addSpiesToCpntSpecFile(targetPath, tree, options, nameString, cpntString);
    sourceFile = (0, utils_1.wmlCreateSourceFile)(targetPath, tree);
    // throw new SchematicsException("stop here")
}
function addNeededImportsToTestFile(sourceFile, targetFilePath, tree) {
    let importsToAdd = [
        {
            packageName: 'rxjs',
            importStatement: `import { of } from 'rxjs';`,
            importMembers: ["of"]
        },
        {
            packageName: "@windmillcode/angular-wml-form",
            importStatement: "import { WMLForm } from '@windmillcode/angular-wml-form';\n",
            importMembers: ["WMLForm"]
        },
    ];
    (0, utils_1.addImportStatements)(sourceFile, targetFilePath, tree, importsToAdd);
}
function addDescribeBlockToCpntSpecFile(targetFilePath, tree, options, nameString, cpntString) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let startNode;
    let targets;
    let optionString = (0, utils_1.createStringObject)(options.name);
    if (!startNode) {
        targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            printChildInfo: false,
            kind: (node, level) => {
                let boolVal0 = node.kind === ts.SyntaxKind.ExpressionStatement;
                // if issues update this regex
                let regexpResult = RegExp(`describe.*['"].*['"]|beforeEach`, "s").exec(node.getFullText());
                let boolVal1 = Boolean(regexpResult);
                let boolVal2;
                if (boolVal1) {
                    // @ts-ignore
                    let binExpr0 = (0, utils_1.wmlFindChildNode)(node, (child) => {
                        let parentDescRegex = RegExp(`describe.*['"].*['"]`, "s");
                        let node = child;
                        let nodesToCheck = Array(level)
                            .fill(null)
                            .map((nullVal, index0) => {
                            node = node.parent;
                            return node;
                        })
                            .filter((node) => {
                            let regexpResult = parentDescRegex.exec(node.getFullText());
                            return node.kind === ts.SyntaxKind.ExpressionStatement &&
                                Boolean(regexpResult);
                        });
                        if (nodesToCheck.length === 1) {
                            // console.log(level)
                        }
                        return nodesToCheck.length === 1;
                    }, "", false, false);
                    boolVal2 = Boolean(binExpr0);
                }
                return boolVal0 && boolVal1 && boolVal2;
            },
        }));
        startNode = targets.at(-1);
    }
    // console.log("0 ",startNode?.getFullText())
    // targets.forEach((x)=>{
    //   getTsMemberInfo(x)
    // })
    if (!startNode) {
        throw new schematics_1.SchematicsException(`the describe block may be missing
    please ensure at a minium the spec file contains
    describe('>>Entity Name <<', () => {

    });

    ${utils_1.WMLschematicMsgs.removePlacehoders0}
    if not contact support if the issue persists
    `);
    }
    let snippet = `\ndescribe('${options.name}ClickSubmitBtn', () => {
    it(\`
    when called |
    under normal conditions |
    does the required action\`, () => {
      // arrange
      const formValue = cpnt.formsService.${cpntString.camelCase()}.${nameString.camelCase(false)}.value; // Your form values go here


      // act
      cpnt.${options.name}ClickSubmitBtn();

      // assert
      // @ts-ignore
      expect(cpnt.${options.apiCall.split("this.")[1]}).toHaveBeenCalledWith(formValue);
      expect(cpnt.baseService.submitForm).toHaveBeenCalled();
    });
  });\n
describe('init${optionString.capitalize(false)}', () => {
    it(\`
    when called |
    under normal conditions |
    does the required action\`, () => {
      // act
      cpnt.init${optionString.capitalize(false)}();

      // assert
      expect(cpnt.${options.name}).toBeInstanceOf(WMLForm);
    });
  });`;
    let spacingPrefix = "\n  ";
    let insertPos = startNode.end;
    (0, utils_1.prepareForUpdate)(snippet, targetFilePath, tree, 
    // @ts-ignore
    insertPos, spacingPrefix);
    return sourceFile;
}
function addSpiesToCpntSpecFile(targetFilePath, tree, options, nameString, cpntString) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let startNode;
    let targets;
    // console.log("2 ",startNode?.getFullText())
    if (!startNode) {
        targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            kind: (node) => {
                let boolVal0 = node.kind === ts.SyntaxKind.ExpressionStatement;
                let regexpResult = RegExp(/fixture.detectChanges(\s*)/).exec(node.getFullText());
                let boolVal1 = Boolean(regexpResult);
                let boolVal2;
                if (boolVal1) {
                    let binExpr0 = (0, utils_1.wmlFindChildNode)(node, ts.SyntaxKind.CallExpression, "", false, false);
                    boolVal2 = Boolean(binExpr0);
                }
                return boolVal0 && boolVal1 && boolVal2;
            },
        }));
        startNode = targets.at(-1);
    }
    // console.log("3 ",startNode?.getFullText())
    if (!startNode) {
        targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            printChildInfo: true,
            kind: (node) => {
                let isKind = node.kind === ts.SyntaxKind.ExpressionStatement;
                let regexpResult = RegExp(`wmlTestUtils.grabComponentInstance\\(\\s*${cpntString.capitalize(false)}\\s*\\)`).exec(node.getFullText());
                let stringContainingConfigure = Boolean(regexpResult);
                let isBinExpr0;
                if (stringContainingConfigure) {
                    let binExpr0 = (0, utils_1.wmlFindChildNode)(node, ts.SyntaxKind.ParenthesizedExpression, "", true, false);
                    isBinExpr0 = Boolean(binExpr0);
                }
                return isKind && stringContainingConfigure && isBinExpr0;
            },
        }));
        startNode = targets.at(0);
    }
    // console.log("4 ",startNode?.getFullText())
    // console.log("5 ",startNode?.getFullText())
    if (!startNode) {
        throw new schematics_1.SchematicsException(`component must be instantiated like so
    ${utils_1.WMLschematicMsgs.removePlacehoders0}

    await wmlTestUtils.configureTestingModuleForStandaloneComponents(>> COMPONENT NAME HERE <<);
    ({fixture, cpnt} =  wmlTestUtils.grabComponentInstance(>> COMPONENT NAME HERE <<));
    `);
    }
    let [, serviceName, seviceMethodName] = options.apiCall.split(".");
    let snippet = `\nspyOn(cpnt.${serviceName}, '${seviceMethodName}');
spyOn(cpnt.baseService,"submitForm")`;
    let spacingPrefix = "\n  ";
    let insertPos = startNode.end;
    (0, utils_1.prepareForUpdate)(snippet, targetFilePath, tree, 
    // @ts-ignore
    insertPos, spacingPrefix);
    return sourceFile;
}
//# sourceMappingURL=index.js.map