"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const change_1 = require("@schematics/angular/utility/change");
const find_module_1 = require("@schematics/angular/utility/find-module");
const parse_name_1 = require("@schematics/angular/utility/parse-name");
const validation_1 = require("@schematics/angular/utility/validation");
const workspace_1 = require("@schematics/angular/utility/workspace");
const utils_1 = require("../utils/utils");
function addImportToNgModule(options) {
    return (host) => {
        if (!options.module) {
            return host;
        }
        const modulePath = options.module;
        const sourceText = host.readText(modulePath);
        const source = ts.createSourceFile(modulePath, sourceText, ts.ScriptTarget.Latest, true);
        const relativePath = (0, utils_1.buildRelativeModulePath)(options, modulePath);
        const changes = (0, ast_utils_1.addImportToModule)(source, modulePath, schematics_1.strings.classify(`${options.name}Module`), relativePath);
        const recorder = host.beginUpdate(modulePath);
        for (const change of changes) {
            if (change instanceof change_1.InsertChange) {
                recorder.insertLeft(change.pos, change.toAdd);
            }
        }
        host.commitUpdate(recorder);
        return host;
    };
}
function default_1(options) {
    return (host) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        if (options.path === undefined) {
            options.path = yield (0, workspace_1.createDefaultPath)(host, options.project);
        }
        if (options.module) {
            // @ts-ignore
            options.module = (0, find_module_1.findModuleFromOptions)(host, options);
        }
        let routingModulePath;
        const isLazyLoadedModuleGen = !!(!!(options.route === "" ? true : options.route) && options.module);
        if (isLazyLoadedModuleGen) {
            routingModulePath = (0, utils_1.getRoutingModuleOrRoutesPath)(host, options.module);
        }
        const parsedPath = (0, parse_name_1.parseName)(options.path, options.name);
        options.name = parsedPath.name;
        options.path = parsedPath.path;
        (0, validation_1.validateClassName)(schematics_1.strings.classify(options.name));
        options.name = options.name.replace(/^wml-/, "WML-");
        (_a = options.componentName) !== null && _a !== void 0 ? _a : (options.componentName = options.name);
        options.componentName = options.componentName.replace(/^wml-/, "WML-");
        const templateSource = (0, schematics_1.apply)((0, schematics_1.url)('./files'), [
            options.routing || (isLazyLoadedModuleGen && routingModulePath)
                ? (0, schematics_1.noop)()
                : (0, schematics_1.filter)((path) => !path.endsWith('-routing.module.ts.template')),
            (0, schematics_1.applyTemplates)(Object.assign(Object.assign(Object.assign({}, schematics_1.strings), { 'if-flat': (s) => (options.flat ? '' : s), lazyRoute: isLazyLoadedModuleGen, lazyRouteWithoutRouteModule: isLazyLoadedModuleGen && !routingModulePath, lazyRouteWithRouteModule: isLazyLoadedModuleGen && !!routingModulePath }), options)),
            (0, schematics_1.move)(parsedPath.path),
        ]);
        const moduleDasherized = schematics_1.strings.dasherize(options.name);
        const modulePath = (0, core_1.normalize)(`${!options.flat ? moduleDasherized + '/' : ''}${moduleDasherized}.module.ts`);
        const componentOptions = {
            module: modulePath,
            flat: options.flat,
            name: options.componentName,
            path: options.path + "/" + options.name,
            project: options.project,
            isPageModule: true,
            route: options.route,
            routeKey: options.routeKey
        };
        return (0, schematics_1.chain)([
            !isLazyLoadedModuleGen ? addImportToNgModule(options) : (0, schematics_1.noop)(),
            !options.fromLibrary ? (0, utils_1.addRouteDeclarationToNgModule)(options, routingModulePath) : (0, schematics_1.noop)(),
            (0, schematics_1.mergeWith)(templateSource),
            isLazyLoadedModuleGen && !options.fromLibrary ? (0, schematics_1.schematic)('component', componentOptions) : (0, schematics_1.noop)()
        ]);
    });
}
exports.default = default_1;
//# sourceMappingURL=index.js.map