"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addRouteDeclarationToModuleOrRoutesFileOnChildrenProperty = void 0;
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const change_1 = require("@schematics/angular/utility/change");
const utils_1 = require("../utils/utils");
function addRouteDeclarationToModuleOrRoutesFileOnChildrenProperty(source, fileToAdd, routeLiteral, module) {
    const routerModuleExpr = (0, ast_utils_1.getRouterModuleDeclaration)(source);
    if (routerModuleExpr) {
        const scopeConfigMethodArgs = routerModuleExpr.arguments;
        if (!scopeConfigMethodArgs.length) {
            const { line } = source.getLineAndCharacterOfPosition(routerModuleExpr.getStart());
            throw new Error(`The router module method doesn't have arguments ` + `at line ${line} in ${fileToAdd}`);
        }
        let routesArray;
        const routesArg = scopeConfigMethodArgs[0];
        // Check if the route declarations array is
        // an inlined argument of RouterModule or a standalone variable
        if (ts.isArrayLiteralExpression(routesArg)) {
            routesArray = routesArg;
        }
        else {
            const routesVarNames = ["children", "routes"];
            let routesVar;
            if (routesArg.kind === ts.SyntaxKind.Identifier) {
                routesVar = source.statements.filter(ts.isVariableStatement).find((v) => {
                    return routesVarNames.includes(v.declarationList.declarations[0].name.getText());
                });
            }
            if (!routesVar) {
                const { line } = source.getLineAndCharacterOfPosition(routesArg.getStart());
                throw new Error(`No route declaration array was found that corresponds ` +
                    `to router module at line ${line} in ${fileToAdd}`);
            }
            routesArray = (0, ast_utils_1.findNodes)(routesVar, ts.SyntaxKind.ArrayLiteralExpression, 1)[0];
        }
        const occurrencesCount = routesArray.elements.length;
        const text = routesArray.getFullText(source);
        let route = routeLiteral;
        let insertPos = routesArray.elements.pos;
        if (occurrencesCount > 0) {
            const lastRouteLiteral = [...routesArray.elements].pop();
            const lastRouteIsWildcard = ts.isObjectLiteralExpression(lastRouteLiteral) &&
                lastRouteLiteral.properties.some((n) => ts.isPropertyAssignment(n) &&
                    ts.isIdentifier(n.name) &&
                    n.name.text === 'path' &&
                    ts.isStringLiteral(n.initializer) &&
                    n.initializer.text === '**');
            const indentation = text.match(/\r?\n(\r?)\s*/) || [];
            const routeText = `${indentation[0] || ' '}${routeLiteral}`;
            // Add the new route before the wildcard route
            // otherwise we'll always redirect to the wildcard route
            if (lastRouteIsWildcard) {
                insertPos = lastRouteLiteral.pos;
                route = `${routeText},`;
            }
            else {
                insertPos = lastRouteLiteral.end;
                route = `,${routeText}`;
            }
        }
        return new change_1.InsertChange(fileToAdd, insertPos, route);
    }
    else if (fileToAdd.endsWith('routes.ts')) {
        let routesArray;
        if (!module) {
            routesArray = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
                node: source,
                printChildInfo: false,
                kind: (node, level) => {
                    let parent = node === null || node === void 0 ? void 0 : node.parent;
                    if (node.kind === ts.SyntaxKind.ArrayLiteralExpression && (parent === null || parent === void 0 ? void 0 : parent.kind) === ts.SyntaxKind.VariableDeclaration && (parent === null || parent === void 0 ? void 0 : parent.getFullText().includes("Routes"))) {
                        return true;
                    }
                    return false;
                }
            }));
        }
        else {
            routesArray = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
                node: source,
                printChildInfo: false,
                kind: (node, level) => {
                    var _a, _b;
                    let ancestors = [(_a = node === null || node === void 0 ? void 0 : node.parent) === null || _a === void 0 ? void 0 : _a.parent, node === null || node === void 0 ? void 0 : node.parent];
                    let nodeFullText = node.getFullText();
                    if (((_b = ancestors[0]) === null || _b === void 0 ? void 0 : _b.kind) === ts.SyntaxKind.ObjectLiteralExpression && ancestors[1].kind === ts.SyntaxKind.PropertyAssignment && ancestors[1].getFullText().trimStart().startsWith("children") && node.kind === ts.SyntaxKind.ArrayLiteralExpression) {
                        let correctEntry = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
                            node: ancestors[0],
                            kind: (node, level) => {
                                let nodeFullText = node.getFullText();
                                if (node.kind === ts.SyntaxKind.PropertyAssignment && (nodeFullText.trimStart().startsWith("loadComponent") || nodeFullText.trimStart().startsWith("loadChildren")) && nodeFullText.includes(module)) {
                                    return true;
                                }
                                return false;
                            }
                        }));
                        if (correctEntry.length > 0) {
                            return true;
                        }
                        debugger;
                        return false;
                    }
                    return false;
                }
            }));
        }
        if (!routesArray) {
            throw new Error("The routes file does not have a routes array or a children array at the target entry please provide one or file a bug report if you are certain the routesArray exists");
        }
        routesArray = routesArray[0];
        const occurrencesCount = routesArray.elements.length;
        const text = routesArray.getFullText(source);
        let route = routeLiteral;
        let insertPos = routesArray.elements.pos;
        if (occurrencesCount > 0) {
            const lastRouteLiteral = [...routesArray.elements].pop();
            const lastRouteIsWildcard = ts.isObjectLiteralExpression(lastRouteLiteral) &&
                lastRouteLiteral.properties.some((n) => ts.isPropertyAssignment(n) &&
                    ts.isIdentifier(n.name) &&
                    n.name.text === 'path' &&
                    ts.isStringLiteral(n.initializer) &&
                    n.initializer.text === '**');
            const indentation = text.match(/\r?\n(\r?)\s*/) || [];
            const routeText = `${indentation[0] || ' '}${routeLiteral}`;
            // Add the new route before the wildcard route
            // otherwise we'll always redirect to the wildcard route
            if (lastRouteIsWildcard) {
                insertPos = lastRouteLiteral.pos;
                route = `${routeText},`;
            }
            else {
                insertPos = lastRouteLiteral.end;
                route = `,${routeText}`;
            }
        }
        return new change_1.InsertChange(fileToAdd, insertPos, route);
    }
    else {
        throw new Error(`No valid routing module or routes.ts file found in ${fileToAdd}`);
    }
}
exports.addRouteDeclarationToModuleOrRoutesFileOnChildrenProperty = addRouteDeclarationToModuleOrRoutesFileOnChildrenProperty;
//# sourceMappingURL=my_helper.js.map