"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFn = exports.TemplateScrollParamsSchema = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const listFiles_1 = require("../utils/listFiles");
const core_1 = require("@angular-devkit/core");
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const utils_1 = require("../utils/utils");
class TemplateScrollParamsSchema {
    constructor() {
        this.envFilePath = utils_1.DevEnvFile;
    }
}
exports.TemplateScrollParamsSchema = TemplateScrollParamsSchema;
let importsToAdd = [
    {
        packageName: "@core/utility/scroll-utils",
        importStatement: "import { ScrollPageParams } from '@core/utility/scroll-utils';\n",
        importMembers: ["ScrollPageParams"]
    },
    {
        packageName: "@angular/core",
        importStatement: "import { ViewChild, ViewContainerRef } from '@core/utility/scroll-utils';\n",
        importMembers: ["ViewChild", "ViewContainerRef"]
    },
];
function generateFn(options) {
    return (tree) => {
        var _a, _b, _c;
        (_a = options.scrollContainerName) !== null && _a !== void 0 ? _a : (options.scrollContainerName = schematics_1.strings.camelize(options.name + "_Scroll_Container"));
        (_b = options.serviceMethodName) !== null && _b !== void 0 ? _b : (options.serviceMethodName = options.name);
        let serviceClassVar = getServiceInfo(tree, options);
        let serviceStringName = (0, utils_1.createStringObject)((_c = serviceClassVar.name) === null || _c === void 0 ? void 0 : _c.getText(), "Service");
        return (0, schematics_1.chain)([
            options.createServiceMethod ? (0, schematics_1.externalSchematic)('@windmillcode/angular-templates', 'serviceMethod', {
                useDataSourceStrategy: options.serviceMethodUseDataSourceStrategy,
                name: options.serviceMethodName,
                envFilePath: options.envFilePath,
                path: options.serviceFilePath,
                serviceName: serviceStringName.camelCase(true),
                apiRoute: options.serviceMethodApiRoute,
                scrollName: options.name,
                transformationType: "scrolling",
                entityName: options.serviceMethodEntityName,
                httpMethod: "post"
            }) : (0, schematics_1.noop)(),
            updateClassFile(options),
        ]);
    };
}
exports.generateFn = generateFn;
let updateClassFile = (options) => {
    return (tree) => {
        let files = (0, listFiles_1.listFiles)(tree, options.path);
        let targetFiles = [
            { suffix: "component.ts", predicate: updateCpntFile },
            { suffix: "service.ts", predicate: updateCpntFile },
            { suffix: "directive.ts", predicate: updateCpntFile },
        ];
        let targetFileName = files.find((file) => {
            return targetFiles.some((val) => file.endsWith(val.suffix));
        });
        let targetFileInfo = targetFiles.find((val) => {
            return targetFileName.endsWith(val.suffix);
        });
        let targetFilePath = (0, core_1.normalize)(`${targetFileName}`);
        targetFileInfo === null || targetFileInfo === void 0 ? void 0 : targetFileInfo.predicate(options, tree, targetFilePath);
    };
};
function updateCpntFile(options, tree, targetFilePath) {
    var _a, _b;
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let cpntClassVar = (0, utils_1.getCpntClassVar)(sourceFile);
    let targetStringName = (0, utils_1.createStringObject)((_a = cpntClassVar.name) === null || _a === void 0 ? void 0 : _a.getText());
    let lastProperty = cpntClassVar.members
        .filter(ts.isPropertyDeclaration)
        .at(-1);
    if (!lastProperty) {
        lastProperty = (0, utils_1.wmlFindChildNode)(cpntClassVar, ts.SyntaxKind.OpenBraceToken, `its with ${targetStringName.orig} class open brace token is it missing`);
    }
    let nameString = (0, utils_1.createStringObject)(options.name);
    let serviceClassVar = getServiceInfo(tree, options);
    let serviceStringName = (0, utils_1.createStringObject)((_b = serviceClassVar.name) === null || _b === void 0 ? void 0 : _b.getText(), "Service");
    let serviceMethodStringName = (0, utils_1.createStringObject)(options.serviceMethodName);
    let serviceMethodScrollPage = nameString.camelCase(true, "ScrollPageObj");
    let serviceMethodScrollPageUIRequestBody = nameString.camelCase(true, "ScrollPageUIRequestBody");
    importsToAdd.push({
        packageName: options.serviceFilePath,
        importStatement: `import { ${serviceStringName.orig} } from '${options.serviceFilePath}/${serviceStringName.lowercase(true)}.service';\n`,
        importMembers: [serviceStringName.orig]
    });
    updateTargetProperties(options, targetFilePath, tree, lastProperty, cpntClassVar);
    updateTargetMethods(targetFilePath, tree, serviceStringName, serviceMethodScrollPage, serviceMethodScrollPageUIRequestBody, serviceMethodStringName, targetStringName, options);
    (0, utils_1.addImportStatements)(sourceFile, targetFilePath, tree, importsToAdd);
    addToNgOnInit(sourceFile, targetFilePath, tree, cpntClassVar, nameString);
    addServiceToConstructor(sourceFile, targetFilePath, tree, cpntClassVar, serviceStringName);
    // sourceFile = wmlCreateSourceFile(targetFilePath, tree);
    // console.log(sourceFile.getFullText())
    // throw new SchematicsException("got here")
}
function addServiceToConstructor(sourceFile, targetFilePath, tree, cpntClassVar, serviceStringName) {
    sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    cpntClassVar = (0, utils_1.getCpntClassVar)(sourceFile);
    (0, utils_1.performDependencyInjection)(targetFilePath, tree, `public ${serviceStringName.camelCase(false)}:${serviceStringName.orig}`, cpntClassVar);
}
function addToNgOnInit(sourceFile, targetFilePath, tree, cpntClassVar, nameString) {
    var _a, _b;
    sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    cpntClassVar = (0, utils_1.getCpntClassVar)(sourceFile);
    let ngOnInit = (0, utils_1.wmlFindChildNode)(cpntClassVar.members, (prop) => {
        var _a;
        return prop.kind === ts.SyntaxKind.MethodDeclaration && ((_a = prop.name) === null || _a === void 0 ? void 0 : _a.getText()) === "ngOnInit";
    }, "ngOnInit is not in this class", false, false);
    if (!ngOnInit) {
        let target = (_b = (_a = cpntClassVar.members
            .filter((prop) => {
            let arrFn = Array.from(prop.getChildren()).filter((child) => {
                child.kind === ts.SyntaxKind.ArrowFunction;
            });
            // for some odd reason, it cant find ArrowFn even when its the case
            return (prop.kind === ts.SyntaxKind.PropertyDeclaration && arrFn.length === 0);
        })
            .at(-1)) !== null && _a !== void 0 ? _a : (0, utils_1.wmlFindChildNode)(cpntClassVar.members, ts.SyntaxKind.Constructor, "its with the constructor", false, false)) !== null && _b !== void 0 ? _b : (0, utils_1.wmlFindChildNode)(cpntClassVar, ts.SyntaxKind.OpenBraceToken, "its with component class close {", false, true);
        let ngOnInitStr = `\n
    ngOnInit(): void{
      this.init${nameString.capitalize(false)}GetScrollEntities() _add_newline_here_
    }
    `;
        (0, utils_1.prepareForUpdate)(ngOnInitStr, targetFilePath, tree, target.end, cpntClassVar.getFullText());
    }
    else {
        let block = (0, utils_1.wmlFindChildNode)(ngOnInit, ts.SyntaxKind.Block, "its with ngOnInit's block");
        let openBrace = (0, utils_1.wmlFindChildNode)(block, ts.SyntaxKind.OpenBraceToken, "its with ngOnInit's open brace");
        (0, utils_1.prepareForUpdate)(`\nthis.init${nameString.capitalize(false)}GetScrollEntities() _add_newline_here_`, targetFilePath, tree, openBrace.end, ngOnInit.getFullText(), "\t\t");
    }
    return { sourceFile, cpntClassVar };
}
function updateTargetProperties(options, targetFilePath, tree, lastProperty, cpntClassVar) {
    let propertiesSnippet = `
@ViewChild("${options.scrollContainerName}",{read:ViewContainerRef,static:true}) ${options.scrollContainerName}
`;
    (0, utils_1.prepareForUpdate)(propertiesSnippet, targetFilePath, tree, lastProperty.end, cpntClassVar.getFullText());
}
function updateTargetMethods(targetFilePath, tree, serviceStringName, serviceMethodScrollPage, serviceMethodScrollPageUIRequestBody, serviceMethodStringName, targetStringName, options) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let cpntClassVar = (0, utils_1.getCpntClassVar)(sourceFile);
    let nameString = (0, utils_1.createStringObject)(options.name);
    let fnsSnippet = `
on${nameString.capitalize(false)}ItemsRetrievedListener = ()=>{
}
init${nameString.classify(false)}GetScrollEntities = ()=>{
  this.${serviceStringName.camelCase(false)}.${serviceMethodScrollPage}?.destroy()
  this.${serviceStringName.camelCase(false)}.${serviceMethodScrollPageUIRequestBody}.pageNum = 0
  this.${serviceStringName.camelCase(false)}.${serviceMethodScrollPage} = new ScrollPageParams({
    pageRequestBody:this.${serviceStringName.camelCase(false)}.${serviceMethodScrollPageUIRequestBody},
    elementToDetermineXFromBottom:this.${options.scrollContainerName}.element.nativeElement,
    ngUnsub:this.ngUnsub,
    cdref:this.cdref,
    callAPIForMoreItemsPredicate:this.${serviceStringName.camelCase(false)}._${serviceMethodStringName.camelCase(false)},
    onItemsRetrievedListener:this.on${nameString.capitalize(false)}ItemsRetrievedListener
  })
  this.cdref.detectChanges()
}
`;
    let lastMethod = cpntClassVar.members
        .filter(ts.isMethodDeclaration)
        .filter((prop) => {
        return ["ngOnInit"].includes(prop.name.getText());
    })
        .at(0);
    let insertPos = lastMethod === null || lastMethod === void 0 ? void 0 : lastMethod.pos;
    if (!lastMethod) {
        lastMethod = (0, utils_1.wmlFindChildNode)(cpntClassVar, ts.SyntaxKind.CloseBraceToken, `its with ${targetStringName.orig} class close brace token is it missing`);
        insertPos = lastMethod.pos;
    }
    (0, utils_1.prepareForUpdate)(fnsSnippet, targetFilePath, tree, insertPos, cpntClassVar.getFullText());
}
// aux
function getServiceInfo(tree, options) {
    let files = (0, listFiles_1.listFiles)(tree, options.serviceFilePath);
    let serviceFileName = files.find((file) => {
        return file.endsWith("service.ts");
    });
    let serviceFilePath = (0, core_1.normalize)(`${serviceFileName}`);
    let serviceSourceFile = (0, utils_1.wmlCreateSourceFile)(serviceFilePath, tree);
    let serviceClassVar = (0, utils_1.getServiceClassVar)(serviceSourceFile);
    return serviceClassVar;
}
//# sourceMappingURL=index.js.map